#include <vdr/plugin.h>

static const char *VERSION        = "0.1.0";
static const char *DESCRIPTION    = "Bridge between VDR and Qiv";
static const char *MAINMENUENTRY  = "VDR2Qiv";

// --- cLineGame -------------------------------------------------------------

class cLineGame : public cOsdObject {
private:
  cOsdBase *osd;
  int x;
  int y;
  char qivcmd;
  eDvbColor color;
public:
  cLineGame(void);
  ~cLineGame();
  virtual void Show(void);
  virtual eOSState ProcessKey(eKeys Key);
  virtual void WriteToFile(void);
  };

cLineGame::cLineGame(void)
{
  osd = NULL;
  x = y = 50;
  color = clrRed;
}

cLineGame::~cLineGame()
{
  SystemExec("pkill qiv_vdr");	/* qiv_vdr beenden */
  delete osd;
}

void cLineGame::Show(void)
{
  osd = cOsd::OpenRaw(100, 100);
  if (osd) {
     osd->Create(0, 0, 500, 360, 4);
     osd->AddColor(clrBackground);

     /* kludgy */
     osd->Text(0, 0, "-- [ Plugin: VDR2Qiv ] --", clrWhite, clrBackground);
     osd->Text(0, 40, "   Rechts", clrWhite, clrBackground);
     osd->Text(0, 70, "   Links", clrWhite, clrBackground);
     osd->Text(0,105, "   Grn", clrWhite, clrBackground);
     osd->Text(0,135, "   Rot", clrWhite, clrBackground);
     osd->Text(0,170, "   OK", clrWhite, clrBackground);
     osd->Text(0,200, "   Gelb", clrWhite, clrBackground);
     osd->Text(0,235, "   FastFwd", clrWhite, clrBackground);
     osd->Text(0,265, "   FastRew", clrWhite, clrBackground);
     osd->Text(0,300, "   Blau", clrWhite, clrBackground);
     osd->Text(0,330, "   Play", clrWhite, clrBackground);

     /* kludgy! */
     osd->Text(120, 40, "weiter", clrWhite, clrBackground);
     osd->Text(120, 70, "zurck", clrWhite, clrBackground);
     osd->Text(120,105, "rechtsherum drehen", clrWhite, clrBackground);
     osd->Text(120,135, "linksherum drehen", clrWhite, clrBackground);
     osd->Text(120,170, "maximieren", clrWhite, clrBackground);
     osd->Text(120,200, "Slideshow", clrWhite, clrBackground);
     osd->Text(120,235, "5 weiter", clrWhite, clrBackground);
     osd->Text(120,265, "5 zurck", clrWhite, clrBackground);
     osd->Text(120,300, "Plugin beenden", clrWhite, clrBackground);
     osd->Text(120,330, "Plugin neustarten", clrWhite, clrBackground);

     osd->Text(240, 40, "Hoch", clrWhite, clrBackground);
     osd->Text(240, 70, "Runter", clrWhite, clrBackground);
     osd->Text(320, 40, "hineinzoomen", clrWhite, clrBackground);
     osd->Text(320, 70, "hinauszoomen", clrWhite, clrBackground);

     osd->Text(370, 230, "< 4   6 >", clrWhite, clrBackground);
     osd->Text(400, 210, " 2", clrWhite, clrBackground);
     osd->Text(400, 180, " /\\", clrWhite, clrBackground);
     osd->Text(400, 250, " 8", clrWhite, clrBackground);
     osd->Text(400, 280, " \\/", clrWhite, clrBackground);



     osd->Flush();
     }
}

eOSState cLineGame::ProcessKey(eKeys Key)
{
  eOSState state = cOsdObject::ProcessKey(Key);
  if (state == osUnknown) {
     switch (Key & ~k_Repeat) {
	/* it's already a hack, so let's hack it some
	 * more - magic chars for some keys */

       case kRight:  qivcmd = ' '; break;  // +1
       case kLeft:   qivcmd = 'j'; break;  // Backspace: -1
       case kFastRew:  qivcmd = 'a'; break;  // PageUp: -5
       case kFastFwd:  qivcmd = 'v'; break;  // PageDown: +5

       case kRed:    qivcmd = 'l'; break;  // Rotate left
       case kGreen:  qivcmd = 'k'; break;  // Rotate right
       case kYellow: qivcmd = 's'; break;  // Slide show

       case kOk:     qivcmd = 'm'; break;  // Maxpect
       case kUp:     qivcmd = '+'; break;  // Zoom in
       case kDown:   qivcmd = '-'; break;  // Zoom out

       case k2:      qivcmd = '2'; break;  // Up: Viewport bewegen
       case k8:      qivcmd = '8'; break;  // Down: Viewport bewegen
       case k4:      qivcmd = '4'; break;  // Left: Viewport bewegen
       case k6:      qivcmd = '6'; break;  // Right: Viewport bewegen

       case kBlue:   return osEnd; break;  // Plugin beenden

       case kPlay:			   // qiv_vdr neustarten
		     qivcmd = 'm';
		     SystemExec("/usr/local/bin/vdr2qiv.sh &");
		     break;

       default: return state;
       }
     WriteToFile();			/* Befehl in die Pipe-Datei schreiben */
     SystemExec("pkill -USR2 qiv_vdr");	/* qiv_vdr benachrichtigen */
     state = osContinue;
     }
  return state;
}

void cLineGame::WriteToFile(void)
{
	FILE *f=fopen("/video/vdr2qiv.pipe", "w");
	if (f)
	{
		fprintf(f, "%c", qivcmd);
		fclose(f);
	}
}

// --- cPluginVdr2qiv --------------------------------------------------------

class cPluginVdr2qiv : public cPlugin {
private:
  // Add any member variables or functions you may need here.
public:
  cPluginVdr2qiv(void);
  virtual ~cPluginVdr2qiv();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return DESCRIPTION; }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Start(void);
  virtual void Housekeeping(void);
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  };

cPluginVdr2qiv::cPluginVdr2qiv(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
}

cPluginVdr2qiv::~cPluginVdr2qiv()
{
  // Clean up after yourself!
}

const char *cPluginVdr2qiv::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return NULL;
}

bool cPluginVdr2qiv::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  return true;
}

bool cPluginVdr2qiv::Start(void)
{
  // Start any background activities the plugin shall perform.
  return true;
}

void cPluginVdr2qiv::Housekeeping(void)
{
  // Perform any cleanup or other regular tasks.
}

cOsdObject *cPluginVdr2qiv::MainMenuAction(void)
{
  // Perform the action when selected from the main VDR menu.
  SystemExec("/usr/local/bin/vdr2qiv.sh &");
  return new cLineGame;
}

cMenuSetupPage *cPluginVdr2qiv::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return NULL;
}

bool cPluginVdr2qiv::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  return false;
}

VDRPLUGINCREATOR(cPluginVdr2qiv); // Don't touch this!
